/*
 * This file is part of SubLink.
 * Copyright (C) 2012  Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

"use strict";

var Cu = Components.utils;
var Cc = Components.classes;
var Ci = Components.interfaces;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://gre/modules/Services.jsm");

var m_justInstalled = false;
var m_startupFinished = false;
var m_location = null;


function install(data, reason) {
  switch (reason) {
    case ADDON_INSTALL:
      m_justInstalled = true;
      Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService).logStringMessage('ADDON_INSTALL');
      break;
    case ADDON_UPGRADE:
      Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService).logStringMessage('ADDON_UPGRADE');
      break;
    case ADDON_DOWNGRADE:
      Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService).logStringMessage('ADDON_DOWNGRADE');
      break;
    default:
      Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService).logStringMessage('??????????????? '+reason);
      break;
  }

}


function uninstall(data, reason) {
  Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService).logStringMessage('uninstall');
}


function startup(data, reason) {
  m_location = data.resourceURI; // file: or jar:
  //m_winCallback = new WindowCallback();
  Services.ww.registerNotification(m_winCallback);

  //var mm = Cc["@mozilla.org/globalmessagemanager;1"].getService(Ci.nsIMessageListenerManager);
  //Services.mm.loadFrameScript("resource://sublink-migreme-4.1.2.348.18-59/content-framescript.js?v=1", true);


  if (reason === APP_STARTUP) {
    // do not delay the startup beyond the necessary
    // remaining work will be done later by onLoadWin => startupFinish
    return;
  }

  startupFinish();

  var winEnum = Services.wm.getEnumerator("navigator:browser");
  while (winEnum.hasMoreElements()) {
    enableWindow(winEnum.getNext().document);
  }
}


function startupFinish() {
  if (m_startupFinished) {
    return;
  }

  m_startupFinished = true;

  registerResourceProtocol(m_location);
  m_location = null;
  Cu.import("resource://sublink-migreme-4.1.2.348.18-59/util.js");

  registerButton();

  if (m_justInstalled) {
    // remove prefs (from previous versions)
    var prefs = Services.prefs.getBranch("extensions.migre.me@hultmann.");
    prefs.deleteBranch("");

    // remove Library item (from previous versions)
    var ns = {};
    Cu.import("resource://sublink-migreme-4.1.2.348.18-59/redir-storage.js", ns);
    ns.redirStorage.removeOrganizerItem();
  }
}


function shutdown(data, reason) {
  if (reason === APP_SHUTDOWN) {
    return;
  }

/*
  var mm = Services.mm;//Cc["@mozilla.org/globalmessagemanager;1"].getService(Ci.nsIMessageListenerManager);
  mm.addMessageListener("migre.me@hultmann:content-disabled", function unload(message) {
    console.log("sublink disable :");
    mm.removeMessageListener("bbb@hultmann:content-disabled", unload);
    registerResourceProtocol(null);
    console.log("sublink disable done!");
  });

  mm.removeDelayedFrameScript("resource://sublink-migreme-4.1.2.348.18-59/content-framescript.js?v=1");
  mm.broadcastAsyncMessage("bbb@hultmann:disable");
*/



  Services.ww.unregisterNotification(m_winCallback);
  m_winCallback = null;

  Cu.import("resource:///modules/CustomizableUI.jsm", {})
    .CustomizableUI.destroyWidget("sublink-migreme-button");


  var winEnum = Services.wm.getEnumerator("navigator:browser");
  while (winEnum.hasMoreElements()) {
    disableWindow(winEnum.getNext().document);
  }


  // unload js modules
  var modules = [
    "api-migreme.js",
    "util.js",
    "util2.js",
    "err.js",
    "run-input.js",
    "redir-storage.js",
    "resolve-redir.js",
    "menu-resolve.js",
    "menu-create.js",
    "button.js",
    "menupopup.js",
    "first-run.js"
  ];
  for (var idx = modules.length - 1; idx > -1; idx--) {
    Cu.unload("resource://sublink-migreme-4.1.2.348.18-59/" + modules[idx]);
  }

  // unregister resource://...
  registerResourceProtocol(null);
}


function registerButton() {
  var jsm = {};
  Cu.import("resource:///modules/CustomizableUI.jsm", jsm);
  jsm.CustomizableUI.createWidget({
    id: "sublink-migreme-button",
    type: "custom",
    defaultArea: jsm.CustomizableUI.AREA_NAVBAR,
    onBuild: doc => {
      var button = doc.createElement("toolbarbutton");
      button.setAttribute("id", "sublink-migreme-button");
      button.setAttribute("class", "toolbarbutton-1 chromeclass-toolbar-additional");
      button.setAttribute("label", "SubLink migre.me");
      button.setAttribute("command", "cmd_sublink-migreme-copy-url");
      button.setAttribute("tooltiptext", util.getText("button.create.label", "migre.me"));
      button.style.MozBinding = "url(chrome://sublink-migreme-4.1.2.348.18-59/content/bindings.xml#toolbarbuttonSmall)";
      return button;
    }
  });
}


function registerResourceProtocol(uri) {
  var module = uri ? Services.io.newURI(uri.resolve("modules/"), null, null) : null;
  Services.io.getProtocolHandler("resource")
    .QueryInterface(Components.interfaces.nsIResProtocolHandler)
    .setSubstitution("sublink-migreme-4.1.2.348.18-59", module);
}


var m_winCallback = {
  QueryInterface: XPCOMUtils.generateQI([Components.interfaces.nsIObserver]),
  observe: function(win, topic, data) {
    if (topic === "domwindowopened") {
      win.addEventListener("load", onLoadWin, false);
    }
  }
};


function onLoadWin(evt) {
  var win = evt.currentTarget;
  win.removeEventListener("load", onLoadWin, false);
  if (win.location.href !== "chrome://browser/content/browser.xul") {
    return;
  }

  if (m_startupFinished) {
    enableWindow(win.document);
  } else {
    // startup in progress...
    win.setTimeout(function(doc) {
      startupFinish();
      enableWindow(doc);
    }, 60, win.document);
  }

  win.addEventListener("unload", onUnloadWin, false);
}


function onUnloadWin(evt) {
  var win = evt.currentTarget;
  win.removeEventListener("unload", onUnloadWin, false);
  disableWindow(win.document);
}


function enableWindow(doc) {
  doc.addEventListener("popupshowing", onPopupShowing, false);
  doc.addEventListener("popupshown", onPopupShown, false);
  Cu.import("resource://sublink-migreme-4.1.2.348.18-59/button.js", {}).addCommandToDom(doc);
}


function disableWindow(doc) {
  doc.removeEventListener("popupshowing", onPopupShowing, false);
  doc.removeEventListener("popupshown", onPopupShown, false);
  Cu.import("resource://sublink-migreme-4.1.2.348.18-59/button.js", {}).removeCommandFromDom(doc);
}


function onPopupShowing(evt) {
  var popup = evt.originalTarget;

  switch (popup.localName) {
    case "tooltip":
      var ttnode = popup.ownerDocument.tooltipNode;
      if (ttnode === null) {
        return; // node from another process
      }
      if (ttnode.id === "sublink-migreme-button") {
        Cu.import("resource://sublink-migreme-4.1.2.348.18-59/button.js", {}).buttonTooltip(popup);
      }
      break;

    case "menupopup":
    case "popup":
      //console.log("onPopupShowing triggerNode =", popup.triggerNode);

      if (isContextMenu(popup)) {
        Cu.import("resource://sublink-migreme-4.1.2.348.18-59/menupopup.js", {}).cmdPopupshowing(popup, evt.target);
      }
      break;
  }
}


// workaround ==> XUL ==> on TwitterFox, document.popupNode in "popupshowing" is not reliable
function onPopupShown(evt) {
  var popup = evt.originalTarget;

  //console.log("onPopupShown triggerNode =", popup.triggerNode);

  switch (popup.localName) {
    case "menupopup":
    case "popup":
      if (isContextMenu(popup)) {
        Cu.import("resource://sublink-migreme-4.1.2.348.18-59/menupopup.js");
        cmdPopupshown(popup);
      }
      break;
  }
}


function isContextMenu(popup) {
  switch (popup.parentNode.localName) {
    case "menu":
    case "toolbarbutton":
      return false;
    //case "popupset":
  }
  return true;
}
