/*
 * This file is part of SubLink.
 * Copyright (C) 2009  Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

"use strict";

var EXPORTED_SYMBOLS = ["redirStorage"];

var ORGANIZER_QUERY_ANNO      = "PlacesOrganizer/OrganizerQuery";
var ANNOTATION_LEFTPANE_VALUE = "hultmann/migre.me/organizer";
var ANNOTATION_REDIR_NAME     = "hultmann/migre.me/urls";


Components.utils.import("resource://sublink-migreme-4.1.2.348.18-59/util.js");


function service() {
  return Cc["@mozilla.org/browser/annotation-service;1"].getService(Ci.nsIAnnotationService);
}


var redirStorage = {

  createOrganizerFolder: function(winOrganizer) {
    var as = service();
    var bs = Cc["@mozilla.org/browser/nav-bookmarks-service;1"].getService(Ci.nsINavBookmarksService);

    var itemId = bs.createFolder(winOrganizer.PlacesUIUtils.leftPaneFolderId,
                                 "migre.me",
                                 bs.DEFAULT_INDEX);
    console.log("createOrganizerFolder =", itemId);
    as.setItemAnnotation(itemId,
                         ORGANIZER_QUERY_ANNO, ANNOTATION_LEFTPANE_VALUE,
                         0, as.EXPIRE_NEVER);
    as.setItemAnnotation(itemId,
                         winOrganizer.EXCLUDE_FROM_BACKUP_ANNO, 1,
                         0, as.EXPIRE_NEVER);
    return itemId;
  },


  removeOrganizerItem: function() {
    var id = this.getOrganizerItemId();
    if (id !== null) {
      Cc["@mozilla.org/browser/nav-bookmarks-service;1"]
        .getService(Ci.nsINavBookmarksService)
        .removeItem(id);
    }
  },


  getOrganizerItemId: function() {
    var as = service();
    var items = as.getItemsWithAnnotation(ORGANIZER_QUERY_ANNO, {});

    for (var idx = 0; idx < items.length; idx++) {
      var val = as.getItemAnnotation(items[idx], ORGANIZER_QUERY_ANNO);
      if (val === ANNOTATION_LEFTPANE_VALUE) {
        return items[idx];
      }
    }
    return null;
  },

  getPagesWithShortUrl: function() {
    // may take few seconds
    return service().getPagesWithAnnotation(ANNOTATION_REDIR_NAME, {});
  },

  loadShortUrls: function(longUri) {
    var serv = service();
    if (serv.pageHasAnnotation(longUri, ANNOTATION_REDIR_NAME) === false) {
      return {};
    }

    var a = serv.getPageAnnotation(longUri, ANNOTATION_REDIR_NAME);
    try {
      return JSON.parse(a);
    } catch (ex) {
      console.log("loadShortUrls: json annotation failed", a);
      return {};
    }
  },


  saveShortUrl: function(longUri, shortUri) {
    var places = Cc["@mozilla.org/browser/history;1"].getService(Ci.mozIAsyncHistory);
    places.isURIVisited(longUri, function(myUri, isVisited) {
      if (isVisited) {
        redirStorage._saveShortUrlCore(longUri, shortUri);
        return;
      }
      places.updatePlaces({
        uri: myUri,
        visits:[{
          visitDate: Date.now() * 1000,
          transitionType: Ci.nsINavHistoryService.TRANSITION_LINK
      }]}, {
        handleResult: function(placeInfo) { // [xpconnect wrapped mozIPlaceInfo]
          console.log('placeInfo',placeInfo);
          redirStorage._saveShortUrlCore(longUri, shortUri);
        }
      });
    });
  },


  _saveShortUrlCore: function(longUri, shortUri) {
    var shortUrl = shortUri.spec;
    var shortUrls = this.loadShortUrls(longUri);

    if (shortUrls.hasOwnProperty(shortUrl)) {
      console.log("saveShortUrl skip", shortUrl);
      return;
    }

    shortUrls[shortUrl] = {
      //"clicks-api":
      //"htmlTitle-api"
      //"created-api"
      //contentType
      "dateAdded": new Date()   //{added:"2009-10-27T00:51:17.604Z"}
    };


    var serv = service();
    try {
      serv.setPageAnnotation(longUri,
                             ANNOTATION_REDIR_NAME,
                             JSON.stringify(shortUrls),
                             0,
                             serv.EXPIRE_WITH_HISTORY);
    } catch (ex) {
      // pages not in history (e.g. about:mozilla) will throw an exception
      console.log("setPageAnnotation EXC", ex);
    }
  },


  removeShortUrl: function(longUri, shortUri) {
    var shortUrl = shortUri.spec;
    var shortUrls = this.loadShortUrls(longUri);

    if (shortUrls.hasOwnProperty(shortUrl) === false) {
      console.log("removeShortUrl skip", shortUrl);
      return false;
    }

    console.log("removeShortUrl before", shortUrls.toSource());
    delete shortUrls[shortUrl];

    var serv = service();
    if (shortUrls.__count__ > 0) {
      console.log("removeShortUrl after", shortUrls.toSource());
      serv.setPageAnnotation(longUri,
                             ANNOTATION_REDIR_NAME,
                             JSON.stringify(shortUrls),
                             0,
                             serv.EXPIRE_WITH_HISTORY);
    } else {
      serv.removePageAnnotation(longUri, ANNOTATION_REDIR_NAME);
      console.log("removeShortUrl after removed!");
    }
    return true;
  },


  getSavedRedir: function(longUri) {
    var json = this.loadShortUrls(longUri);
    for (var shortUrl in json) {
      if (json.hasOwnProperty(shortUrl)) {
        return shortUrl;
      }
    }
    return null;
  }

};
